<?php if ( ! defined('CARTTHROB_PATH')) Cartthrob_core::core_error('No direct script access allowed');
/**
 * @package     Cartthrob_discount_buy_a_get_b
 * @author      Dorothy Molloy
 * @link 		http://www.climbingturn.co.uk/
 * @copyright 	Copyright (c) 2014, Climbing Turn Ltd
 * @license 	LGPL v3
 *
 *  This file is part of Cartthrob_discount_buy_a_get_b.

 Cartthrob_discount_buy_a_get_b is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Cartthrob_discount_buy_a_get_b is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the Lesser GNU General Public License
 along with Cartthrob_discount_buy_a_get_b.  If not, see <http://www.gnu.org/licenses/>.
 */

class Cartthrob_discount_buy_a_get_b extends Cartthrob_discount
{
	public $title = 'buy_a_get_b';

	public $settings = array(
			array(
					'name' => 'buy_a_entry_ids',
					'short_name' => 'a_entry_ids',
					'note' => 'separate_multiple_entry_ids',
					'type' => 'text'
			),
			array(
					'name' => 'buy_a_quantity',
					'short_name' => 'buy_a_purchase_quantity',
					'note' => 'enter_the_purchase_quantity',
					'type' => 'text'
			),
			array(
					'name' => 'valid_b',
					'short_name' => 'b_entry_ids',
					'note' => 'separate_multiple_entry_ids',
					'type' => 'text'
			),
			array(
					'name' => 'num_b_per_a',
					'short_name' => 'num_b_discounted_per_num_a_required',
					'note' => 'enter_the_number_of_items',
					'type' => 'text'
			),
			array(
					'name' => 'percentage_off',
					'short_name' => 'percentage_off',
					'note' => 'enter_the_percentage_discount',
					'type' => 'text'
			),
			array(
					'name' => 'amount_off',
					'short_name' => 'amount_off',
					'note' => 'enter_the_discount_amount',
					'type' => 'text'
			),
			array(
					'name' => 'per_item_limit',
					'short_name' => 'item_limit',
					'note' => 'per_item_limit_note',
					'type' => 'text'
			),
	);

	private $a_entry_ids = array();
	private $percentage_off;
	private $amount_off = 0;
	private $b_entry_ids = array();
	private $max_num_of_b_permitted = 0;
	private $proportional_num_b = 1;
	private $num_a = 0;
	private $num_b = 0;
	
	public function initialize($plugin_settings = array()){
		ee()->lang->loadfile('Cartthrob_discount_buy_a_get_b', 'Cartthrob');
		parent::initialize($plugin_settings);
	}

	function get_discount()
	{
		$this->get_b_ids();
		$this->get_a_ids();
		$this->num_b = $this->number_in_cart($this->b_entry_ids);
		$this->num_a = $this->number_in_cart($this->a_entry_ids);
		$this->calculate_percentage_or_amount_off();
		$this->max_num_of_b_permitted = ( $this->plugin_settings('item_limit') ) ? $this->plugin_settings('item_limit') : FALSE;
		$items = $this->create_array_of_discountable_items();
		if (isset($items)){
			$discount = $this->calculate_total_discount($items);
		}
		return $discount;
	}

	private function get_a_ids(){
		if ( $this->plugin_settings('a_entry_ids') )
		{
			$this->a_entry_ids = preg_split('/\s*[,|]\s*/', trim( $this->plugin_settings('a_entry_ids') ));
		}
		return count($this->a_entry_ids);
	}
	

	private function get_b_ids(){
		if ($this->plugin_settings('b_entry_ids'))
		{
			$this->b_entry_ids = preg_split('/\s*[,|]\s*/', $this->plugin_settings('b_entry_ids'));
		}
	}
	
	
	private function calculate_percentage_or_amount_off(){
		if ($this->plugin_settings('percentage_off') !== '')
		{
			$this->percentage_off = $this->core->sanitize_number( $this->plugin_settings('percentage_off') );

			if ($this->percentage_off > 100)
			{
				$this->percentage_off = 100;
			}
			else if ($this->percentage_off < 0)
			{
				$this->percentage_off = 0;
			}
		}
		else
		{
			$this->amount_off = $this->core->sanitize_number( $this->plugin_settings('amount_off') );
		}
	}
	
	private function number_in_cart($a_or_b_array){
		$num = 0;	
		foreach ($this->core->cart->items() as $item)
		{
			if (in_array($item->product_id(), $a_or_b_array))
			{
				$num += $item->quantity();
			}
		}
		return $num;
	}

	private function create_array_of_discountable_items(){
		$items;
		foreach ($this->core->cart->items() as $item)
		{
			$prices = $this->extrapolate_prices_from_b($item, $this->plugin_settings('b_entry_ids'));
			if (isset($prices)){
				foreach($prices as $price){
					$items[] = $price;
				}
			}
		}
		return $items;
	}


	private function extrapolate_prices_from_b($item, $entry_ids){
		$items;
		$settings = explode(",", $entry_ids);
		if ( $item->product_id() && in_array( $item->product_id(), $settings))
		{
			for ($i=0; $i<$item->quantity() ;$i++)
			{
				$items[] = $item->price();
			}
			return $items;
		}
	}
	
	private function calculate_amount($original_item_price){
		$discount = 0;
		if ($this->percentage_off > 0){
			$discount += $original_item_price * $this->percentage_off / 100;
		}else{
			$discount = $this->amount_off;
		}
		return $discount;
	}

	private function calculate_total_discount(&$items){
 		$num_a_required = $this->plugin_settings('buy_a_purchase_quantity');

 		$this->proportional_num_b = ($this->plugin_settings('num_b_discounted_per_num_a_required'))? $this->plugin_settings('num_b_discounted_per_num_a_required'): 1;
		$num_b_accrued = ((int)($this->num_a/$num_a_required) * $this->proportional_num_b);
		
		if ($this->max_num_of_b_permitted){
			//number of accrued discounted items may not exceed the max permitted in the settings
			$max_b_exceeded = false;
			while ($num_b_accrued > $this->max_num_of_b_permitted){
				$max_b_exceeded = true;
				$num_b_accrued--;
			}
			if ($max_b_exceeded) $this->set_error( $this->core->lang('coupon_limit_msg') );
		}
		
 		$discount = 0;
		while($num_b_accrued > 0)
		{
			$last_item = array_pop($items);
			if (isset($last_item)){
				$discount += $this->calculate_amount($last_item);
				$num_b_accrued --;
			}
		}
		return $discount;
	}


	/*
	 * Check that this is a valid discount
	*/
	function validate()
	{
		$this->get_b_ids();
		$this->num_b = $this->number_in_cart($this->b_entry_ids);
		if ($this->num_b == 0) return FALSE;
		if ($this->get_a_ids() == 0) return FALSE;
		
		$this->calculate_percentage_or_amount_off();
		if ($this->amount_off == 0 && $this->percentage_off == 0){
			$this->set_error( $this->core->lang('coupon_minimum_not_reached') );
		}
		
		$this->num_a = $this->number_in_cart($this->a_entry_ids);
		$num_a_required = $this->plugin_settings('buy_a_purchase_quantity');
		if (!count($this->a_entry_ids) >= $num_a_required){
			$this->set_error( $this->core->lang('coupon_minimum_not_reached') );
			return FALSE;
		}
		return TRUE;
	}
}